/*
 * Simple RC2/40-CBC encryptor.
 *
 * Copyright 1997, Chris Hall - Counterpane Systems
 *
 * This program is very simple.  It takes an input file, encrypts it using
 * a specified key and IV with RC2/40-CBC, and writes the result to an
 * output file.  The output file contains the IV in the first 8 bytes and
 * the ciphertext in the remaining bytes.  This is meant to be given to
 * some program like search.c (included with this archive).
 */

#include <stdio.h>

void usage(char *progname)
{
    fprintf(stderr,
	"usage: %s <input file> <output file> <iv> <key>\n",
	progname);
}

unsigned char hex(char c)
{
    if (c >= '0' && c <= '9')
	return c - '0';
    if (c >= 'A' && c <= 'Z')
	return c - 'A' + 10;
    if (c >= 'a' && c <= 'z')
	return c - 'a' + 10;
    return 0xFF;
}

#define xorbuf(d, s) \
do { \
    (d)[0] ^= (s)[0]; \
    (d)[1] ^= (s)[1]; \
    (d)[2] ^= (s)[2]; \
    (d)[3] ^= (s)[3]; \
} while (0)

int main(int argc, char **argv)
{
    unsigned char   key[128], IV[8];
    unsigned short  p[4], c[4], *iv;
    FILE    *input, *output;
    short   i;
    long    len;

    if (argc != 5) {
	usage(argv[0]);
	return -1;
    }

    if ((input = fopen(argv[1], "rb")) == NULL) {
	fprintf(stderr, "error opening %s for input.\n", argv[1]);
	perror(argv[0]);
	return -1;
    }
    if ((output = fopen(argv[2], "wb")) == NULL) {
	fprintf(stderr, "error opening %s for output.\n", argv[2]);
	perror(argv[0]);
	return -1;
    }

    /* read the IV from the command-line */
    if (strlen(argv[3]) != 16) {
	fprintf(stderr, "invalid IV.\n");
	usage(argv[0]);
	return -1;
    }
    for (i = 0; i < 8; i++)
	IV[i] = (hex(argv[3][2*i]) << 4) | hex(argv[3][2*i+1]);

    /* read the encryption key from the command-line */
    if (strlen(argv[4]) != 10) {
	fprintf(stderr, "invalid IV.\n");
	usage(argv[0]);
	return -1;
    }
    for (i = 0; i < 5; i++)
	key[i] = (hex(argv[4][2*i]) << 4) | hex(argv[4][2*i+1]);

    fseek(input, 0L, SEEK_END);
    len = ftell(input);
    rewind(input);

    rc2_full_keyschedule(key);

    fwrite(IV, 1, 8, output);
    iv = (unsigned short*)IV;

    while (len > 0) {
	memset(p, 0, sizeof(p));
	fread(p, 1, 8, input);
	xorbuf(p, iv);
	rc2_encrypt(key, p, c);
	memcpy(iv, c, 8);
	fwrite(c, 1, 8, output);
	len -= 8;
    }

    fclose(input);
    fclose(output);

    return 0;
}
